/* Aranha Configuration management
 *
 * arch-tag: 5c36d9e4-cdd3-4a0b-947e-c8c7ec0af4b5
 *
 * Copyright Daniel Silverstone <dsilvers@digital-scurf.org>
 * Copyright (C) 2004-2005 Pepperfish Ltd
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "config.h"
#include "configurator.h"
#include "interpreters.h"
#include "utils.h"
#include <string.h>

static lua_State *
configstate()
{
  static lua_State *_configstate = NULL;
  if( ! _configstate ) {
    _configstate = prepare_full_interpreter(ICTX_CONFIG);
    if( ! _configstate ) {
      /* Something scary and vile happened; bail bail bail! */
      BAIL("Unable to prepare a configuration state\n");
    }
  }
  return _configstate;
}
#define CONFIGSTATE lua_State *L = configstate()

/* Fill out target with the value of the number specified by name */
int 
get_config_int( const char* name, int *target)
{
  CONFIGSTATE;
  lua_settop(L,0);
  lua_pushstring(L,name);
  lua_gettable(L, LUA_GLOBALSINDEX);
  if( ! lua_isnumber(L,1) )
    return 1;
  *target = (int)(lua_tonumber(L,1));
  return 0;
}

/* Fill out target with a copy of the string specified by name. It is
 * the responsibility of the caller to free that string once they're done
 * with it
 */
int 
get_config_string( const char* name, char **target)
{
  CONFIGSTATE;
  lua_settop(L,0);
  lua_pushstring(L,name);
  lua_gettable(L, LUA_GLOBALSINDEX);
  if( ! lua_tostring(L,1) )
    return 1;
  *target = strdup(lua_tostring(L,1));
  return 0;
}
