/* Aranha Interpreter management
 *
 * arch-tag: a6481b38-5a87-40b5-b3cc-b366bf9f4be4
 *
 * Copyright Daniel Silverstone <dsilvers@digital-scurf.org>
 * Copyright (C) 2004-2005 Pepperfish Ltd
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "interpreters.h"
#include "lualib.h"
#include "lauxlib.h"
#include "fastcgi/lua_fastcgi.h"

#include <stdlib.h>
#include <unistd.h>

static int 
L_getenv(lua_State *L)
{
  const char* envname = luaL_checkstring(L,1);
  if( getenv(envname) )
    lua_pushstring(L, getenv(envname));
  return (getenv(envname))?1:0;
}

static int
L_getcwd(lua_State *L)
{
  char buffer[PATH_MAX];
  lua_pushstring(L, getcwd(buffer, PATH_MAX));
  return 1;
}

lua_State *
prepare_full_interpreter(int context)
{
  lua_State *L = lua_open();
  
  /* All contexts load the full complement of standard libraries */
  luaopen_stdlibs(L); lua_settop(L,0);
  /* All contexts load the full complement of lua startup code */
#include "luastartup.inc"
  lua_settop(L,0);
  
  lua_register(L,"getenv",L_getenv);
  lua_register(L,"getcwd",L_getcwd);
  
  if( context == ICTX_CMDLINE ) {
#include "startup/utils.cinc"
#include "startup/modules.cinc"
#include "startup/cmdline.cinc"
  } else if(context == ICTX_CONFIG ) {
#include "startup/config.cinc"
  } else if(context == ICTX_ARANHA ) {
    /* Load the FastCGI library in first */
    if( InitialiseFastCGILibrary(L) != 0) {
      lua_close(L);
      return 0;
    }
    {
#include "startup/utils.cinc"
#include "startup/modules.cinc"
#include "startup/aranha.cinc"
    }
  } else {
    /* Erm, fuck! */
    lua_close(L);
    return 0;
  }
  
  /* Finally return the interpreter */
  return L;
}
