/* Aranha Main code
 *
 * arch-tag: 8b26068d-dc47-4547-a985-155af3272876
 *
 * Copyright Daniel Silverstone <dsilvers@digital-scurf.org>
 * Copyright (C) 2004-2005 Pepperfish Ltd
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "config.h"
#include "fastcgi/lua_fastcgi.h"
#include "lauxlib.h"
#include "interpreters.h"
#include "utils.h"
#include "configurator.h"
#include "parentchild.h"

#include <stdlib.h>

#include <unistd.h>

static int
commandline_invoke( int argc,
                    char **argv )
{
  /* We were invoked as a commandline tool, find the first argument
   * as a command from COMMANDTOOL_PATH and load and run it
   */
  lua_State *L = prepare_full_interpreter(ICTX_CMDLINE);
  int ret = 0, i;
  /* With our commandline interpreter ready at full-pelt... */
  lua_getglobal(L,"_TRACEBACK");
  lua_getglobal(L,"main");
  for(i = 0; i < argc; ++i) {
    lua_pushstring(L,argv[i]);
  }
  if( lua_pcall(L,argc,1,1) > 0 ) {
    if( lua_tostring(L,-1) ) {
      fprintf(stderr,"ARANHA RUNTIME ERROR: %s\n",
              lua_tostring(L,-1));
      return 1;
    }
  }
  ret = lua_tonumber(L,-1);
  lua_close(L);
  return ret;
}


static int
fastcgi_invoke()
{
  /* We are invoked as a fastcgi, prepare the parent and run? */
  PrepareParent();
  ParentLoop();
  return 0;
}

int 
main( int argc,
      char **argv )
{
  if( argc == 1 ) {
    /* No arguments, check if stdin is a tty */
    if( isatty(0) ) {
      /* STDIN is a tty, pretend we were invoked as
       * aranha usage
       */
      char* fake_argv[2] = {
        NULL,
        "usage" };
      fake_argv[0] = argv[0];
      return commandline_invoke( 2, fake_argv );
    } else {
      /* No arguments, and stdin is not a tty */
      return fastcgi_invoke();
    }
  }
  return commandline_invoke(argc, argv);
}
