/*
** $Id: llimits.h,v 1.65 2005/03/09 16:28:07 roberto Exp $
** Limits, basic types, and some other `installation-dependent' definitions
** See Copyright Notice in lua.h
*/

#ifndef llimits_h
#define llimits_h


#include <limits.h>
#include <stddef.h>


#include "lua.h"


#define api_check	luai_apicheck
  
  
typedef LUAI_UINT32 lu_int32;
  
typedef LUAI_UMEM lu_mem;

typedef LUAI_MEM l_mem;
  



/*
** signed/unsigned integer big enough to count the total memory used by Lua;
** it should be at least as large as `size_t'.
** Note: size_t is defined to be the result of sizeof(), i.e. the size of
** a single memory object; but we need to account for multiple memory objects.
** Alas, this is only an issue on 16 bit platforms with the LP32 data model.
*/
#if LUA_BITSINT >= 32
/* 32 and 64 bit platforms */
/* using size_t is safe. using ptrdiff_t is lame, but ssize_t is not in C89 */
/* do not use long here as this will break on LLP64 platforms (WIN64) */
#define MAXLMEM	((~(lu_mem)0)>>1)
#else
/* 16 bit platforms */
#define MAXLMEM	LUA_MAXINT32
#endif


/* chars used as small naturals (so that `char' is reserved for characters) */
typedef unsigned char lu_byte;


#define MAX_SIZET	((size_t)(~(size_t)0)-2)

#define MAX_LUMEM	((lu_mem)(~(lu_mem)0)-2)


#define MAX_INT (INT_MAX-2)  /* maximum value of an int (-2 for safety) */

/*
** conversion of pointer to integer
** this is for hashing only; there is no problem if the integer
** cannot hold the whole pointer value
*/
#define IntPoint(p)  ((unsigned int)(lu_mem)(p))



/* type to ensure maximum alignment */
typedef LUAI_USER_ALIGNMENT_T L_Umaxalign;


/* result of a `usual argument conversion' over lua_Number */
typedef LUAI_UACNUMBER l_uacNumber;


#define check_exp(c,e)	(lua_assert(c), (e))


#ifndef UNUSED
#define UNUSED(x)	((void)(x))	/* to avoid warnings */
#endif


#ifndef cast
#define cast(t, exp)	((t)(exp))
#endif



/*
** type for virtual-machine instructions
** must be an unsigned with (at least) 4 bytes (see details in lopcodes.h)
*/
typedef lu_int32 Instruction;



/* maximum stack for a Lua function */
#define MAXSTACK	250



/* minimum size for the string table (must be power of 2) */
#ifndef MINSTRTABSIZE
#define MINSTRTABSIZE	32
#endif


/* minimum size for string buffer */
#ifndef LUA_MINBUFFER
#define LUA_MINBUFFER	32
#endif


#endif
